/**
 *
 */
package edu.agh.sm2006.bluetooth;

import java.io.IOException;
import java.util.Vector;

import javax.bluetooth.BluetoothStateException;
import javax.bluetooth.DeviceClass;
import javax.bluetooth.DiscoveryAgent;
import javax.bluetooth.DiscoveryListener;
import javax.bluetooth.LocalDevice;
import javax.bluetooth.RemoteDevice;
import javax.bluetooth.ServiceRecord;
import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.StringItem;
import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

/**
 * Main midlet's class
 *
 * @author Kuba Madej
 *
 */
public class BluetoothDiscovery extends MIDlet implements DiscoveryListener,
		CommandListener {
	/**
	 * Error dialog label
	 */
	private static final String ERROR_ALERT_LABEL = "Error";

	/**
	 * Discovery agent
	 */
	private DiscoveryAgent discoveryAgent;

	/**
	 * Midlet's display
	 */
	private Display display = Display.getDisplay(this);

	/**
	 * Midlet's main and only form
	 */
	private Form form = new Form(null);

	private Vector discoveredDevices = new Vector();

	/**
	 * Exit command
	 */
	private Command exitCommand = new ExecutableCommand("Exit", Command.EXIT,
			99) {

		public void execute() {
			notifyDestroyed();
		}

	};

	/**
	 * Stop discovery command
	 */
	private Command stopDiscoveryCommand = new ExecutableCommand(
			"Stop discovery", Command.OK, 1) {

		public void execute() {
			discoveryAgent.cancelInquiry(BluetoothDiscovery.this);
		}

	};

	/**
	 * Start discovery command
	 */
	private Command startDiscoveryCommand = new ExecutableCommand(
			"Start discovery", Command.OK, 1) {

		public void execute() {
			discoveredDevices.removeAllElements();
			state.toogleState();
			try {
				discoveryAgent.startInquiry(DiscoveryAgent.GIAC,
						BluetoothDiscovery.this);
			} catch (BluetoothStateException e) {
				displayAlert(e);
			}
		}

	};

	/**
	 * Discovery state
	 */
	private DiscoveryState state = new DiscoveryState(form,
			startDiscoveryCommand, stopDiscoveryCommand);

	/**
	 * Default constructor
	 */
	public BluetoothDiscovery() {
		try {
			discoveryAgent = LocalDevice.getLocalDevice().getDiscoveryAgent();
			discoveryAgent.startInquiry(DiscoveryAgent.GIAC, this);
		} catch (BluetoothStateException e) {
			displayAlert(e);
			notifyDestroyed();
		}

		form.addCommand(exitCommand);
		form.setCommandListener(this);
		display.setCurrent(form);
	}

	protected void destroyApp(boolean arg0) throws MIDletStateChangeException {
	}

	protected void pauseApp() {

	}

	protected void startApp() throws MIDletStateChangeException {
	}

	/**
	 * Displays an alert
	 *
	 * @param e
	 *            exception to extract alert's message from
	 */
	private void displayAlert(Exception e) {
		display.setCurrent(new Alert(ERROR_ALERT_LABEL, e.getMessage(), null,
				AlertType.ERROR), form);
	}

	public void deviceDiscovered(RemoteDevice device, DeviceClass arg1) {
		discoveredDevices.addElement(device);
	}

	public void inquiryCompleted(int arg0) {
		for (int i = 0; i < discoveredDevices.size(); i++) {
			final RemoteDevice device = (RemoteDevice) discoveredDevices
					.elementAt(i);
			try {
				form.append(new StringItem(device.getFriendlyName(false),
						device.getBluetoothAddress()));
			} catch (IOException e) {
				displayAlert(e);
			}
		}
		state.toogleState();
	}

	public void serviceSearchCompleted(int arg0, int arg1) {
	}

	public void servicesDiscovered(int arg0, ServiceRecord[] arg1) {
	}

	public void commandAction(Command command, Displayable arg1) {
		if (command instanceof ExecutableCommand)
			((ExecutableCommand) command).execute();
	}
}
