/**
 *
 */
package edu.agh.sm2006.editor;

import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.List;
import javax.microedition.lcdui.TextBox;
import javax.microedition.lcdui.TextField;
import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;
import javax.microedition.rms.RecordEnumeration;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;
import javax.microedition.rms.RecordStoreFullException;
import javax.microedition.rms.RecordStoreNotFoundException;

/**
 * Midlet's main class
 *
 * @author Kuba Madej
 *
 */
public class Editor extends MIDlet implements CommandListener {
	/**
	 * Editors capacity - number of characters it can hold
	 */
	private static final int EDITOR_CAPACITY = 4096;

	/**
	 * Error dialog label
	 */
	private static final String ERROR_ALERT_LABEL = "Error";

	/**
	 * Message displayed when user chooses to load a file and no loadable files
	 * are found
	 */
	private static final String FILE_NOT_FOUND = "No files found";

	/**
	 * Editor's text box
	 */
	private TextBox textBox = new TextBox(null, null, EDITOR_CAPACITY,
			TextField.ANY);

	/**
	 * Text field to enter file name to save
	 */
	private TextField textField = new TextField("Enter file name:", "", 999, 1);

	/**
	 * Save form
	 */
	private Form saveForm = new Form("File to save");

	/**
	 * Load list
	 */
	private List loadList = new List("File to load", List.EXCLUSIVE);

	/**
	 * Midlet's display
	 */
	private Display display = Display.getDisplay(this);

	/**
	 * Opens load dialog
	 */
	private Command displayLoadFormCommand = new EditorCommand("Load",
			Command.SCREEN, 5) {

		public void execute() {
			loadList.deleteAll();

			final String[] fileNames = RecordStore.listRecordStores();
			if (fileNames == null) {
				displayAlert(FILE_NOT_FOUND);
				return;
			}

			for (int i = 0; i < fileNames.length; i++) {
				loadList.append(fileNames[i], null);
			}

			display.setCurrent(loadList);
		}

	};

	/**
	 * Opens save dialog
	 */
	private Command displaySaveFormCommand = new EditorCommand("Save",
			Command.SCREEN, 10) {

		public void execute() {
			display.setCurrent(saveForm);
		}

	};

	/**
	 * Clears text box
	 */
	private Command newCommand = new EditorCommand("New", Command.SCREEN, 1) {

		public void execute() {
			saveForm.addCommand(saveCommand);
			saveForm.removeCommand(loadCommand);
			textBox.setString("");
		}

	};

	/**
	 * Exits
	 */
	private Command exitCommand = new EditorCommand("Exit", Command.EXIT, 99) {

		public void execute() {
			notifyDestroyed();
		}

	};

	/**
	 * Cancels load/save operation
	 */
	private Command cancelCommand = new EditorCommand("Cancel", Command.EXIT,
			99) {

		public void execute() {
			display.setCurrent(textBox);
		}

	};

	/**
	 * Saves the file
	 */
	private Command saveCommand = new EditorCommand("OK", Command.OK, 1) {

		public void execute() {
			try {
				RecordStore store = RecordStore.openRecordStore(textField
						.getString(), true);
				RecordEnumeration enumeration = store.enumerateRecords(null,
						null, false);
				final byte[] toStore = textBox.getString().getBytes();
				final int size = toStore.length;

				if (enumeration.hasNextElement()) {
					store.setRecord(enumeration.nextRecordId(), toStore, 0,
							size);
				} else {
					store.addRecord(toStore, 0, size);
				}

				display.setCurrent(textBox);
			} catch (RecordStoreFullException e) {
				displayAlert(e);
			} catch (RecordStoreNotFoundException e) {
				displayAlert(e);
			} catch (RecordStoreException e) {
				displayAlert(e);
			}
		}

	};

	/**
	 * Loads a file
	 */
	private Command loadCommand = new EditorCommand("OK", Command.OK, 1) {

		public void execute() {
			try {
				final String fileName = loadList.getString(loadList
						.getSelectedIndex());
				final RecordStore store = RecordStore.openRecordStore(fileName,
						false);
				final RecordEnumeration enumeration = store.enumerateRecords(
						null, null, false);
				final byte[] data = enumeration.nextRecord();
				textBox.setString(data != null ? new String(data) : "");
				textField.setString(fileName);
				display.setCurrent(textBox);
			} catch (RecordStoreFullException e) {
				displayAlert(e);
			} catch (RecordStoreNotFoundException e) {
				displayAlert(e);
			} catch (RecordStoreException e) {
				displayAlert(e);
			}
		}

	};

	/**
	 * Construcotr which initializes all GUI components
	 */
	public Editor() {
		textBox.addCommand(displayLoadFormCommand);
		textBox.addCommand(displaySaveFormCommand);
		textBox.addCommand(newCommand);
		textBox.addCommand(exitCommand);
		textBox.setCommandListener(this);

		saveForm.append(textField);
		saveForm.addCommand(cancelCommand);
		saveForm.addCommand(saveCommand);
		saveForm.setCommandListener(this);

		loadList.addCommand(cancelCommand);
		loadList.addCommand(loadCommand);
		loadList.setCommandListener(this);

		display.setCurrent(textBox);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see javax.microedition.midlet.MIDlet#destroyApp(boolean)
	 */
	protected void destroyApp(boolean arg0) throws MIDletStateChangeException {
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see javax.microedition.midlet.MIDlet#pauseApp()
	 */
	protected void pauseApp() {
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see javax.microedition.midlet.MIDlet#startApp()
	 */
	protected void startApp() throws MIDletStateChangeException {
	}

	public void commandAction(Command command, Displayable arg1) {
		if (command instanceof EditorCommand) {
			((EditorCommand) command).execute();
		}
	}

	/**
	 * Displayes alert dialog
	 *
	 * @param e
	 *            exception to extract dialog message from
	 */
	private void displayAlert(Exception e) {
		displayAlert(e.getMessage());
	}

	/**
	 * Displayes alert dialog
	 *
	 * @param message
	 *            message to display
	 */
	private void displayAlert(final String message) {
		display.setCurrent(new Alert(ERROR_ALERT_LABEL, message, null,
				AlertType.ERROR), textBox);
	}
}
