package edu.agh.sm2006.player;

import java.io.IOException;

import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.StringItem;
import javax.microedition.lcdui.TextField;
import javax.microedition.media.Manager;
import javax.microedition.media.MediaException;
import javax.microedition.media.Player;
import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

/**
 * Audio player midlet
 *
 * @author Kuba Madej
 *
 */
public class SoundPlayer extends MIDlet implements CommandListener {
	/**
	 * Error dialog label
	 */
	private static final String ERROR_ALERT_LABEL = "Error";

	/**
	 * Open file command
	 */
	private Command openCommand = new PlayerCommand("Open", Command.SCREEN, 1) {

		public void execute() {
			new Thread() {
				public void run() {
					status.setText(Status.OPENING);
					try {
						final Player oldPlayer = player;
						player = Manager.createPlayer(textField.getString());
						player.prefetch();
						if (oldPlayer != null)
							oldPlayer.stop();
					} catch (IOException e) {
						displayAlert(e);
					} catch (MediaException e) {
						displayAlert(e);
					}
					status.setText(Status.IDLE);
				}
			}.start();
		}

	};

	/**
	 * Exit command
	 */
	private Command exitCommand = new PlayerCommand("Exit", Command.EXIT, 99) {

		public void execute() {
			notifyDestroyed();
		}

	};

	/**
	 * Stop command
	 */
	private Command stopCommand = new PlayerCommand("Stop", Command.SCREEN, 50) {

		public void execute() {
			if (player != null) {
				try {
					player.stop();
				} catch (MediaException e) {
					displayAlert(e);
				}
				status.setText(Status.IDLE);
			}
		}

	};

	/**
	 * Play command
	 */
	private Command playCommand = new PlayerCommand("Play", Command.SCREEN, 40) {

		public void execute() {
			if (player != null) {
				try {
					player.start();
				} catch (MediaException e) {
					displayAlert(e);
				}
				status.setText(Status.PLAYING);
			}
		}

	};

	/**
	 * File to play selection text field
	 */
	private TextField textField = new TextField("File to play:", "", 999,
			TextField.ANY);

	/**
	 * Status label
	 */
	private StringItem status = new StringItem("Status:", Status.IDLE);

	/**
	 * Midlet's display object
	 */
	private Display display;

	/**
	 * Current Player instance
	 */
	private Player player;

	/**
	 * Midlet's form
	 */
	private Form form;

	/**
	 * Flag indicating wheather the player was playing before the application
	 * was paused
	 */
	private boolean wasPlaying;

	/**
	 * Media time at the moment when playback was paused
	 */
	private long mediaTime;

	/**
	 * Default constructor
	 *
	 */
	public SoundPlayer() {
		display = Display.getDisplay(this);
		form = new Form("Player");
		form.append(textField);
		form.append(status);
		form.addCommand(openCommand);
		form.addCommand(stopCommand);
		form.addCommand(exitCommand);
		form.addCommand(playCommand);
		form.setCommandListener(this);
		display.setCurrent(form);
	}

	protected void destroyApp(boolean arg0) throws MIDletStateChangeException {
	}

	protected void pauseApp() {
		if (player != null && player.getState() == Player.STARTED) {
			try {
				mediaTime = player.getMediaTime();
				player.stop();
			} catch (MediaException e) {
				displayAlert(e);
				wasPlaying = false;
				return;
			}
			wasPlaying = true;
		} else
			wasPlaying = false;
	}

	protected void startApp() throws MIDletStateChangeException {
		if (wasPlaying) {
			try {
				player.start();
			} catch (MediaException e) {
				displayAlert(e);
			}
			if (mediaTime != Player.TIME_UNKNOWN)
				try {
					player.setMediaTime(mediaTime);
				} catch (MediaException e) {
					// the media type doesn't support setMediaTime, just ignore
				}
		}
	}

	public void commandAction(Command command, Displayable displayable) {
		if (command instanceof PlayerCommand)
			((PlayerCommand) command).execute();
	}

	/**
	 * Displayes alert
	 *
	 * @param e
	 *            exception to get alert message from
	 */
	private void displayAlert(Exception e) {
		display.setCurrent(new Alert(ERROR_ALERT_LABEL, e.getMessage(), null,
				AlertType.ERROR), form);
		status.setText(Status.IDLE);
	}
}
